//	KaleidoTileVelocities.h

#pragma once

#include "KaleidoTileMatrices.h"

#define PI	3.14159265358979323846


//	An instantaneous velocity is a Lie derivative, which looks like
//
//		( 0   vr -vx )    ( 0   vr  0  )    ( 0   vr  vx )
//		(-vr  0  -vy )    (-vr  0   0  )    (-vr  0   vy )
//		( vx  vy  0  )    ( vx  vy  0  )    ( vx  vy  0  )
//
//	in spherical, Euclidean and hyperbolic geometry, respectively.
//	The instantaneous velocity is well defined independently
//	of the basepoint, but nevertheless it's convenient to interpret
//	it from the point of view of an observer at the north pole (0,0,1)
//	who feels
//
//		vx as a velocity in the x-direction
//		vy as a velocity in the y-direction
//		vr as a rotational velocity.
//
//	This interpretation remains the same in all three geometries,
//	so a velocity can stay the same even when we change geometries.
//
//	Note:  I would have preferred to call the rotational velocity
//	vθ instead of vr, but my compiler doesn't accept the θ character
//	in a field name.

typedef struct
{
	double	x,	//	x-component of velocity, as seen at (0,0,1)
			y,	//	y-component of velocity, as seen at (0,0,1)
			r;	//	rotational component of velocity, as seen at (0,0,1)
} ktVelocity;


extern void	IntegrateVelocity(double aTimeInterval, ktVelocity aVelocity, ktGeometry aGeometry, ktMatrix aMotion);
extern void	DifferentiatePosition(ktMatrix aPosition, ktGeometry aGeometry, ktVelocity *aVelocity);
