//	Matrix44.c
//
//	© 2010 by Jeff Weeks
//	See TermsOfUse.txt

#include "Matrix44.h"


void Matrix44Identity(double m[4][4])
{
	unsigned int	i,
					j;

	for (i = 0; i < 4; i++)
		for (j = 0; j < 4; j++)
			m[i][j] = (i == j ? 1.0 : 0.0);
}


void Matrix44Copy(
	double	dst[4][4],	//	output destination matrix
	double	src[4][4])	//	input  source matrix
{
	unsigned int	i,
					j;

	for (i = 0; i < 4; i++)
		for (j = 0; j < 4; j++)
			dst[i][j] = src[i][j];
}


void Matrix44Product(
	double	m1[4][4],		//	input first  factor
	double	m2[4][4],		//	input second factor
	double	product[4][4])	//	output product, which may coincide with either or both inputs
{
	unsigned int	i,
					j,
					k;
	double			temp[4][4];

	for (i = 0; i < 4; i++)
		for (j = 0; j < 4; j++)
		{
			temp[i][j] =  0.0;
			for (k = 0; k < 4; k++)
				temp[i][j] += m1[i][k] * m2[k][j];
		}

	for (i = 0; i < 4; i++)
		for (j = 0; j < 4; j++)
			product[i][j] = temp[i][j];
}


void Matrix44DoubleToFloat(
	float	dst[4][4],	//	output destination matrix
	double	src[4][4])	//	input  source matrix
{
	unsigned int	i,
					j;

	for (i = 0; i < 4; i++)
		for (j = 0; j < 4; j++)
			dst[i][j] = (float) src[i][j];
}


void Matrix44TimesColumnVector(
	double	m[4][4],	//	input
	double	v[4],		//	input
	double	mv[4])		//	output, OK to pass same array as v
{
	unsigned int	i,
					j;
	double			temp[4];
	
	for (i = 0; i < 4; i++)
	{
		temp[i] = 0.0;
		for (j = 0; j < 4; j++)
			temp[i] += m[i][j] * v[j];
	}

	for (i = 0; i < 4; i++)
		mv[i] = temp[i];
}
