//	KaleidoTileMacOSView.h
//
//	© 2010 by Jeff Weeks
//	See TermsOfUse.txt

#import <Cocoa/Cocoa.h>
#include "KaleidoTile-Common.h"


@interface KaleidoTileMacOSView : NSOpenGLView
{
	//	Preferences Panel

	IBOutlet NSPanel		*itsPreferencesPanel;
	IBOutlet NSTabView		*itsPrefTabView;

	IBOutlet NSBox			*itsPrefGeneralBox;
	IBOutlet NSTextField	*itsPrefWindowWidthLabel,
							*itsPrefWindowWidth;
	IBOutlet NSTextField	*itsPrefWindowDistanceLabel,
							*itsPrefWindowDistance;
	IBOutlet NSBox			*itsPrefStereo3DBox;
	IBOutlet NSTextField	*itsPrefEyeSeparationLabel,
							*itsPrefEyeSeparation,
							*itsPrefEyeSeparationUnits;

	IBOutlet NSButton		*itsPrefFrameRate,
							*itsPrefSyncToMonitor,
							*itsPrefMultisampling,
							*itsPrefShaders;

	IBOutlet NSButton		*itsPrefOK;	
	
	//	Symmetry Group Panel
	IBOutlet NSPanel		*itsSymmetryPanel;
	IBOutlet NSPopUpButton	*itsSymmetryP,
							*itsSymmetryQ,
							*itsSymmetryR;
	IBOutlet NSButton		*itsSymmetryOK;	
	
	//	An NSTimer supports continuous redraw.
	NSTimer					*itsTimer;
	
	//	The Microseconds clock lets us keep the animation smooth
	//	regardless of the frame rate.
	long long				itsLastRenderTime;
	
	//	If we're displaying the frame rate in the title bar,
	//	we'll want to update it only about once per second.
	long long				itsLastFrameRateTime;
	
	//	Cocoa automatically remembers the "last directory" but really
	//	we need a separate "last directory" for each type of file,
	//	for example for the last TIFF image saved.
	NSMutableString			*itsLastImageDirectory;
	
	//	Remember the default window title so we can restore it
	//	after displaying the frame rate.
	NSMutableString			*itsDefaultWindowTitle;
	
	//	Keep a set of custom cursors.
	//	(See comments in initWithFrame.)
	NSCursor				*itsOpenHandCursor,
							*itsClosedHandCursor,
							*itsPointingHandCursor;
	
	//	The ModelData contains all the information about the space itself
	//	along with all platform-independent information about how
	//	it is to be displayed.
	ModelData				md;
}
+ (NSOpenGLPixelFormat*)pixelFormatSupportingMultisampling:(bool)aMultisampleFlag;
- (id)initWithFrame:(NSRect)frameRect;
- (NSCursor *)loadCustomCursor:(NSString *)aCursorName;
- (void)dealloc;
- (void)awakeFromNib;
- (NSRect)windowWillUseStandardFrame:(NSWindow *)window defaultFrame:(NSRect)newFrame;
- (void)applicationDidFinishLaunching:(NSNotification *)aNotification;
- (BOOL)acceptsFirstResponder;
- (void)resetCursorRects;
- (void)setRedrawWithSync:(bool)aSyncFlag;
- (void)pleaseRedraw;
- (void)keyDown:(NSEvent *)anEvent;
- (void)insertText:(id)aString;
- (void)mouseDown:(NSEvent *)anEvent;
- (void)mouseDragged:(NSEvent *)anEvent;
- (void)mouseUp:(NSEvent *)anEvent;
- (NSPoint)mouseLocation:(NSEvent *)anEvent;
- (void)scrollWheel:(NSEvent *)anEvent;
- (void)drawRect:(NSRect)bounds;
- (BOOL)validateMenuItem:(NSMenuItem *)aMenuItem;
- (IBAction)commandExportCopy:(id)sender;
- (IBAction)commandExportSave:(id)sender;
- (IBAction)commandChooseSymmetries:(id)sender;
- (IBAction)updateSymmetryGroup:(id)sender;
- (IBAction)closeSymmetryPanel:(id)sender;
- (IBAction)commandShowPlainImages:(id)sender;
- (IBAction)commandShowReflectedImages:(id)sender;
- (IBAction)commandCutAlongMirrorLines:(id)sender;
- (IBAction)commandMakeTilesBigger:(id)sender;
- (IBAction)commandMakeTilesSmaller:(id)sender;
- (IBAction)commandSnapToArchimedeanSolids:(id)sender;
- (IBAction)commandViewMultisampling:(id)sender;
- (IBAction)commandMesh2:(id)sender;
- (IBAction)commandMesh4:(id)sender;
- (IBAction)commandMesh8:(id)sender;
- (IBAction)commandMesh16:(id)sender;
- (IBAction)commandTilingRadius40:(id)sender;
- (IBAction)commandTilingRadius45:(id)sender;
- (IBAction)commandTilingRadius50:(id)sender;
- (IBAction)commandTilingRadius55:(id)sender;
- (IBAction)commandTilingRadius60:(id)sender;
- (IBAction)commandGraphicsFixedFunction:(id)sender;
- (IBAction)commandGraphicsShaders:(id)sender;
- (IBAction)commandViewStereoNone:(id)sender;
- (IBAction)commandViewStereoAnaglyphic:(id)sender;
- (IBAction)commandViewStereoGeoWall:(id)sender;
- (void)setStereoMode:(StereoMode)aStereoMode;
- (IBAction)commandLanguageCy:(id)sender;
- (IBAction)commandLanguageEn:(id)sender;
- (IBAction)commandLanguageFr:(id)sender;
- (IBAction)commandLanguageIt:(id)sender;
- (IBAction)commandLanguageJa:(id)sender;
- (IBAction)commandLanguagePt:(id)sender;
- (IBAction)commandHelpStudyQuestions:(id)sender;
- (IBAction)setUpPreferencesPanel:(id)sender;
- (void) setStereoLabels;
- (IBAction)readOnePreference:(id)sender;
- (void)setHeightFromWidth;
- (void)replaceOpenGLContext;
- (IBAction)shutDownPreferencesPanel:(id)sender;
- (NSImage *)contentsAsImage;
- (NSData *)contentsAsTIFF;
- (NSBitmapImageRep *)contentsAsBitmapImageRep;
- (void)invertRawImage: (unsigned int *)aPixelBuffer pixelsWide:(unsigned int)aWidth pixelsHigh:(unsigned int)aHeight;
- (void)changeLanguage:(NSString *)aLanguageCode;
- (NSMenu *)buildLocalizedMenuBar;
- (NSMenu *)addToMenu:(NSMenu *)aParentMenu submenuWithTitle:(char *)aSubmenuTitle;
- (void)setPreferencePanelLabels;
- (void)setSymmetryPanelLabels;
- (void)reshape;
- (void)windowWillMiniaturize:(NSNotification *)aNotification;
- (BOOL)windowShouldClose:(id)sender;
- (BOOL)applicationShouldTerminateAfterLastWindowClosed:(NSApplication *)theApplication;
- (NSApplicationTerminateReply)applicationShouldTerminate:(NSApplication *)sender;
- (void)applicationWillTerminate:(NSNotification *)aNotification;
@end
